/**
 * JASMINe
 * Copyright (C) 2006-2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.jade.wrapper.apache;

import java.io.File;
import java.net.InetAddress;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.jasmine.configurator.ConfigurationException;
import org.objectweb.jasmine.configurator.Configurator;
import org.objectweb.jasmine.jade.resource.Resource;
import org.objectweb.jasmine.jade.util.DebugResources;
import org.objectweb.jasmine.jade.util.JadeException;
import org.objectweb.jasmine.jade.util.Logger;
import org.objectweb.jasmine.launcher.Launcher;
import org.ow2.jasmine.jade.fractal.api.control.GenericAttributeController;
import org.ow2.jasmine.jade.fractal.api.control.NoSuchAttributeException;
import org.ow2.jasmine.jade.wrapper.apache.configurator.ConfiguratorApache;
import org.ow2.jasmine.jade.wrapper.apache.launcher.LauncherApacheImpl;

public class ApacheWrapper implements GenericAttributeController, Resource,
        BindingController {

    /**
     *
     */
    String nodeName;
    /**
     * directory to install the Apache HTTP server.
     */
    String dirLocal;
    /**
     *
     */
    Launcher launcher;
    /**
     *
     */
    Configurator configurator;
    /**
     *
     */
    HashMap<String, GenericAttributeController> jonas = new HashMap<String, GenericAttributeController>();

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    public ApacheWrapper() throws JadeException {

        configurator = new ConfiguratorApache();
        launcher = new LauncherApacheImpl(configurator);

        try {
            nodeName = InetAddress.getLocalHost().getHostName();
        } catch (Exception e) {
            throw new JadeException("ApacheResourceImpl: cannot read HostName "
                    + e);
        }
    }

    // ------------------------------------------------------------------------
    // Implementation of the Resource interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.resource.Resource#configure()
     */
    public void configure() throws JadeException {

        Logger.println(DebugResources.on, "[ApacheWrapper] configure2");

        try {
            configurator.configure();
        } catch (ConfigurationException e) {
            throw new JadeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.resource.Resource#loadApp()
     */
    public void loadApp() throws JadeException {
        Logger.println(DebugResources.on, "[ApacheWrapper] loadApp");
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.resource.Resource#start()
     */
    public void start() throws JadeException {

        Logger.println(DebugResources.on, "[ApacheWrapper] start");

        try {
            launcher.start();
        } catch (Exception e) {
            throw new JadeException(e);
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.resource.Resource#stop()
     */
    public void stop() throws JadeException {

        Logger.println(DebugResources.on, "[ApacheWrapper] stop");

        try {
            launcher.stop();
        } catch (Exception e) {
            throw new JadeException(e);
        }
    }

    // -------------------------------------------------------------------------
    // Implementation of GenericAttributeController interface
    // -------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jade.fractal.api.control.GenericAttributeController#setAttribute(java.lang.String,
     *      java.lang.String)
     */
    public void setAttribute(String name, String value)
            throws NoSuchAttributeException {

        if (name.matches("dirLocal")) {
            this.dirLocal = value;
            if (!dirLocal.endsWith(File.separator)) {
                dirLocal += File.separator;
            }
        }

        configurator.setProperty(name, value);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jade.fractal.api.control.GenericAttributeController#getAttribute(java.lang.String)
     */
    public String getAttribute(String name) throws NoSuchAttributeException {
        if (name.equals("dirLocal"))
            return dirLocal;

        return configurator.getProperty(name).toString();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.jasmine.jade.fractal.api.control.GenericAttributeController#listFcAtt()
     */
    public String[] listFcAtt() {

        Set<Object> keys = configurator.getProperties().keySet();

        return keys.toArray(new String[0]);
    }

    // ------------------------------------------------------------------------
    // Implementation of BindingController interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() {
        HashSet hs = new HashSet(jonas.keySet());
        return (String[]) hs.toArray(new String[hs.size()]);
        // return null;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String itfName) throws NoSuchInterfaceException {
        if (itfName.startsWith("jonas")) {
            return (Object) jonas.get(itfName);
        }
         return null;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
     *      java.lang.Object)
     */
    public void bindFc(String itfName, Object itfValue)
            throws NoSuchInterfaceException, IllegalBindingException,
            IllegalLifeCycleException {

        // if (itfName.startsWith("jonas")) {
        // jonas.put(itfName, (GenericAttributeController) itfValue);
        // try {
        // System.out.println("instance-jonas-name "
        // + ((GenericAttributeController) itfValue)
        // .getAttribute("instance-jonas-name"));
        // } catch (Exception e) {
        // e.printStackTrace();
        // }
        // return;
        // }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String itfName) throws NoSuchInterfaceException,
            IllegalBindingException, IllegalLifeCycleException {
        // if (itfName.startsWith("jonas")) {
        // jonas.remove(itfName);
        // return;
    }

}
