package org.ow2.jasmine.jade.wrapper.apache.configurator;

import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;

import org.apache.velocity.VelocityContext;
import org.objectweb.jasmine.configurator.ConfigurationException;
import org.objectweb.jasmine.configurator.statik.velocity.VelocityConfiguratorImpl;

public class WorkersConfigurator extends VelocityConfiguratorImpl {

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    public WorkersConfigurator(VelocityContext vcontext, ClassLoader cl,
            String templateName, String fileName) {
        super(vcontext, cl, templateName, fileName);
    }

    // ------------------------------------------------------------------------
    // Implementation of the AbstractVelocityConfigurator interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.configurator.statik.velocity.AbstractVelocityConfigurator#configure()
     */
    @Override
    public void configure() throws ConfigurationException {

        super.configure();

        /*
         * we must reset workers-list for the next generate and because we want
         * generates are independant.
         */
        this.setProperty("workers-list", new HashSet<String>());
    }

    @Override
    public boolean setProperty(String key, Object value) {

        /*
         * processing if the property is a worker
         */
        if (key.equals("workers")) {
            setWorker(key, value.toString());
            return true;
        } else {
            return super.setProperty(key, value);
        }
    }

    // ------------------------------------------------------------------------
    // Implementation of the interface
    // ------------------------------------------------------------------------

    public void setWorker(String key, String worker) {

        Set<String> workersSet = null;

        StringTokenizer tokenizer = new StringTokenizer(worker, ",");

        Object o = this.getProperty("workers-list");
        if (o == null) {
            workersSet = new HashSet<String>();
        } else {
            if (o instanceof Set) {
                workersSet = (Set<String>) o;
            } else {
                workersSet = new HashSet<String>();
            }
        }

        if (tokenizer.countTokens() > 2) {

            String workerName = tokenizer.nextToken();
            String workerHost = tokenizer.nextToken();
            String workerPort = tokenizer.nextToken();

            // Worker w = new Worker(workerName, workerHost, workerPort);

            workersSet.add(workerName);

            this.setProperty("worker-" + workerName + "-host", workerHost);
            this.setProperty("worker-" + workerName + "-port", workerPort);
            this.setProperty("workers-list", workersSet);

        }
    }

}
