/**
 * JASMINe
 * Copyright (C) 2006-2007 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.jade.wrapper.apache.launcher;

import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.objectweb.jasmine.configurator.Configurator;
import org.objectweb.jasmine.launcher.DisplayThread;
import org.objectweb.jasmine.launcher.LaunchException;
import org.objectweb.jasmine.launcher.Launcher;

/**
 *
 * <p>
 * contributor : Julien Legrand
 * </p>
 *
 * @author Yann Davin <yann.davin@gmail.com>
 *
 */
public class LauncherApacheImpl implements Launcher {

    /**
     *
     */
    static Logger logger = Logger.getLogger(LauncherApacheImpl.class.getName());
    /**
     *
     */
    private Configurator configurator;
    /**
     *
     */
    ApacheThread apacheThread = null;
    /**
     *
     */
    private Boolean started = false;

    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    public LauncherApacheImpl(Configurator configurator) {

        super();

        logger.setLevel(Level.INFO);

        this.configurator = configurator;
    }

    // ------------------------------------------------------------------------
    // Implementation of the LauncherItfc interface
    // ------------------------------------------------------------------------

    public void configure() throws LaunchException {

    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.launcher.Launcher#start()
     */
    public void start() throws LaunchException {
        if (apacheThread == null) {
            apacheThread = new ApacheThread();
        }
        apacheThread.start();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.launcher.Launcher#stop()
     */
    public void stop() throws LaunchException {

        System.out.println("Stop APACHE");

        Runtime r = Runtime.getRuntime();

        try {
            /*
             * Process p = r.exec(getJonasRoot() + "/bin/unix/jonas" + " stop -n " +
             * getInstanceName());
             */

            StringBuffer command = new StringBuffer(configurator
                    .getProperty("instance-apache-home")
                    + "/bin/apachectl stop");
            if (Integer.parseInt(configurator.getProperty("server-listen-port")
                    .toString()) < 1024) {
                command.insert(0, "sudo ");
            }

            Process s = r.exec(command.toString());
            s.waitFor();
            if (apacheThread != null) {
                apacheThread.stopMe();
                apacheThread.join();
                apacheThread = null;
            }
            this.setStarted(false);

        } catch (Exception e) {
            throw new LaunchException(e);
        }
    }

    // ------------------------------------------------------------------------
    // Private methods
    // ------------------------------------------------------------------------

    private Boolean isStarted() {
        return started;
    }

    private void setStarted(Boolean started) {
        this.started = started;
    }

    // ------------------------------------------------------------------------
    // Overriden method
    // ------------------------------------------------------------------------

    protected void finalize() throws Throwable {
        if (this.isStarted()) {
            this.stop();
        }
        super.finalize();
    }

    // ------------------------------------------------------------------------
    // Inner Classes
    // ------------------------------------------------------------------------

    private class ApacheThread extends Thread {

        /**
         *
         */
        boolean stop = false;
        /**
         *
         */
        Process script;
        /**
         * to display error stream of Apache HTTP server
         */
        DisplayThread edt;
        /**
         * to display normal stream of Apache HTTP server
         */
        DisplayThread ndt;

        // ------------------------------------------------------------------------
        // Implementation of the Runnable interface
        // ------------------------------------------------------------------------

        public void run() {

            if (configurator.getProperty("instance-apache-home") == null)
                return;

            Runtime r = Runtime.getRuntime();
            try {
                StringBuffer command = new StringBuffer(configurator
                        .getProperty("instance-apache-home")
                        + "/bin/apachectl start");
                if (Integer.parseInt(configurator.getProperty(
                        "server-listen-port").toString()) < 1024) {
                    command.insert(0, "sudo ");
                }

                script = r.exec(command.toString());

                InputStream inputstream = script.getInputStream();
                ndt = new DisplayThread(inputstream, "[apache http server]");
                ndt.start();

                InputStream errorstream = script.getErrorStream();
                edt = new DisplayThread(errorstream, "[apache http server]");
                edt.start();

                script.waitFor();
                setStarted(true);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        public void stopMe() {
            this.stop = true;
            if (edt != null)
                edt.setStopped(true);
            if (ndt != null)
                ndt.setStopped(true);
            if (script != null) {
                script.destroy();
            }
        }
    }

}
